// FeedSelectDlg.cpp : implementation file
//

#include "stdafx.h"
#include "RSSSubmit.h"
#include "FeedSelectDlg.h"
#include "FeedFunctions.h"
#include "ProgressDlg.h"
#include "ResultsDlg.h"

using namespace std;

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CFeedSelectDlg dialog


CFeedSelectDlg::CFeedSelectDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CFeedSelectDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CFeedSelectDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}


void CFeedSelectDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CFeedSelectDlg)
	DDX_Control(pDX, IDC_VALIDATESELECTION, m_Validate);
	DDX_Control(pDX, IDC_FEEDSELECTPREV, m_Prev);
	DDX_Control(pDX, IDC_SELECTLIST, m_FeedList);
	DDX_Control(pDX, IDC_FEEDSELECTNEXT, m_Next);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CFeedSelectDlg, CDialog)
	//{{AFX_MSG_MAP(CFeedSelectDlg)
	ON_BN_CLICKED(IDC_FEEDSELECTNEXT, OnNext)
	ON_WM_CLOSE()
	ON_WM_ERASEBKGND()
	ON_BN_CLICKED(IDC_VALIDATESELECTION, OnValidateSelection)
	ON_BN_CLICKED(IDC_FEEDSELECTPREV, OnPrev)
	ON_COMMAND(ID_ACTION_BACK, OnActionBack)
	ON_COMMAND(ID_ACTION_NEXT, OnActionNext)
	ON_COMMAND(ID_ACTION_VALIDATE, OnActionValidate)
	ON_COMMAND(ID_FILE_CHECKUPDATES, OnFileCheckupdates)
	ON_COMMAND(ID_FILE_EXIT, OnFileExit)
	ON_COMMAND(ID_LOG_CLEARLOG, OnLogClearlog)
	ON_COMMAND(ID_LOG_VIEWLOG, OnLogViewlog)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CFeedSelectDlg message handlers

void CFeedSelectDlg::OnNext() 
{
	int iItem = m_FeedList.GetNextItem(-1, LVNI_SELECTED);

	if(iItem == -1)
	{
		MessageBox("You need to select a RSS Feed to continue.", "No Selection");
		return;
	}

	iItem = -1;

	gSelectedFeeds.clear();
	while((iItem = m_FeedList.GetNextItem(iItem, LVNI_SELECTED)) != -1)
	{
		gSelectedFeeds.push_back(gFeeds[iItem]);
	}

	EndDialog(2);
}

void CFeedSelectDlg::OnClose() 
{
	EndDialog(0);
	
	CDialog::OnClose();
}

BOOL CFeedSelectDlg::OnEraseBkgnd(CDC* pDC) 
{
	CRect rect;
	GetClientRect(&rect);
	pDC->FillSolidRect(&rect, OUR_BGCOLOR);  // some color
	return FALSE;
}

BOOL CFeedSelectDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();

	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	// Set title
	SetWindowText(gProgramName);

	// Tool Tips
	m_ToolTipCtrl.Create(this);
	m_ToolTipCtrl.AddTool(&m_Next, "Next");
	m_ToolTipCtrl.AddTool(&m_Prev, "Prev");
	m_ToolTipCtrl.AddTool(&m_Validate, "Validate");

	// Menu
	CMenu *menu = GetMenu();
	
	// Add Columns to List Control
	CRect rect;
	m_FeedList.GetClientRect(&rect);
	int nColInterval = rect.Width()/5;

	m_FeedList.InsertColumn(0, _T("Title"), LVCFMT_LEFT, nColInterval*2);
	m_FeedList.InsertColumn(1, _T("URL"), LVCFMT_LEFT, nColInterval*2);
	m_FeedList.InsertColumn(2, _T("Description"), LVCFMT_LEFT, rect.Width()-4*nColInterval);

	// Extra styles: Full Row Select
    DWORD styles = LVS_EX_FULLROWSELECT | LVS_EX_FLATSB|LVS_EX_HEADERDRAGDROP;
    // Use macro since this is new and not in MFC.
	m_FeedList.SetExtendedStyle(styles);

	// Update Feed List
	UpdateFeedList();
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CFeedSelectDlg::UpdateFeedList()
{
	// Load Feeds
	LoadFeeds();
	
	// Clear out old list
	m_FeedList.DeleteAllItems();

	// Repopulate list
	int id = 0;
	for(vector < FEED >::iterator iter = gFeeds.begin(); iter != gFeeds.end(); ++iter)
	{
		m_FeedList.InsertItem(id, iter->title.c_str());
		m_FeedList.SetItem(id, 1, LVIF_TEXT, iter->url.c_str(), 0, 0, 0, 0);
		m_FeedList.SetItem(id, 2, LVIF_TEXT, iter->description.c_str(), 0, 0, 0, 0);

		id++;
	}
}

void CFeedSelectDlg::OnValidateSelection() 
{
	int iItem = m_FeedList.GetNextItem(-1, LVNI_SELECTED);

	if(iItem == -1)
	{
		MessageBox("You need to select a RSS Feed to continue.", "No Selection");
		return;
	}

	iItem = -1;

	gSelectedFeeds.clear();
	while((iItem = m_FeedList.GetNextItem(iItem, LVNI_SELECTED)) != -1)
	{
		gSelectedFeeds.push_back(gFeeds[iItem]);
	}

	// Progress Dialog
	{
		CProgressDlg dlg;
		dlg.ValidateAll();
		int nResponse = dlg.DoModal();

		if(nResponse != 2)
			return;
	}

	// Results Dialog
	{
		CResultsDlg dlg;
		gDisplayValidation = 1;
		int nResponse = dlg.DoModal();
		gDisplayValidation = 0;
	}
}

void CFeedSelectDlg::OnPrev() 
{
	EndDialog(1);
}

BOOL CFeedSelectDlg::PreTranslateMessage(MSG* pMsg) 
{
	m_ToolTipCtrl.RelayEvent(pMsg);
	
	return CDialog::PreTranslateMessage(pMsg);
}

void CFeedSelectDlg::OnActionBack() 
{
	OnPrev();
}

void CFeedSelectDlg::OnActionNext() 
{
	OnNext();
}

void CFeedSelectDlg::OnActionValidate() 
{
	OnValidateSelection();
}

void CFeedSelectDlg::OnFileCheckupdates() 
{
	WinExec("AutoUpdater.exe", SW_SHOW);
}

void CFeedSelectDlg::OnFileExit() 
{
	EndDialog(0);
}

void CFeedSelectDlg::OnLogClearlog() 
{
	ClearLog();
}

void CFeedSelectDlg::OnLogViewlog() 
{
	ViewLog();
}
